ACTION_ITEM = "ACTION_ITEM"
ACTION_ITEM_EXPORT = "ACTION_ITEM_EXPORT"
ACTION_ITEM_VIEW_ONLY = "ACTION_ITEM_VIEW_ONLY"
action_items_codenames = [
    "edc_action_item.add_actionitem",
    "edc_action_item.add_reference",
    "edc_action_item.change_actionitem",
    "edc_action_item.change_reference",
    "edc_action_item.delete_actionitem",
    "edc_action_item.delete_reference",
    "edc_action_item.view_actionitem",
    "edc_action_item.view_actiontype",
    "edc_action_item.view_historicalactionitem",
    "edc_action_item.view_historicalreference",
    "edc_action_item.view_reference",
]

navbar_codenames = [
    "edc_action_item.nav_action_item_section",
]

navbar_tuples = []
for codename in navbar_codenames:
    navbar_tuples.append((codename, f"Can access {codename.split('.')[1]}"))

action_items_codenames.extend(navbar_codenames)



import sys

from django.apps import AppConfig as DjangoAppConfig

from edc_appointment.constants import COMPLETE_APPT

from .timepoint import Timepoint
from .timepoint_collection import TimepointCollection


class AppConfig(DjangoAppConfig):
    name = "edc_timepoint"
    verbose_name = "Edc Timepoint"

    timepoints = TimepointCollection(
        timepoints=[
            Timepoint(
                model="edc_appointment.appointment",
                datetime_field="appt_datetime",
                status_field="appt_status",
                closed_status=COMPLETE_APPT,
            ),
            Timepoint(
                model="edc_appointment.appointment",
                datetime_field="appt_datetime",
                status_field="appt_status",
                closed_status=COMPLETE_APPT,
            ),
        ]
    )

    def ready(self):
        sys.stdout.write(f"Loading {self.verbose_name} ...\n")
        for model in self.timepoints:
            sys.stdout.write(f" * '{model}' is a timepoint model.\n")
        sys.stdout.write(f" Done loading {self.verbose_name}.\n")



# Generated by Django 6.0 on 2026-01-18 17:12

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0099_historicalstock_confirmed_at_location_and_more"),
    ]

    operations = []



from django.utils.translation import gettext_lazy as _

from edc_list_data.model_mixins import ListModelMixin


class InfoSources(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = _("Information Source")
        verbose_name_plural = _("Information Sources")



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from .row import Row

if TYPE_CHECKING:
    from django.apps import AppConfig


class ListModelMakerError(Exception):
    pass


class ListModelMaker:
    def __init__(
        self,
        display_index: int,
        row: tuple[str, str] | Row,
        model_name: str,
        apps: AppConfig | None = None,
    ):
        self.extra_value: str | None = None
        self.custom_name: str | None = None

        self.display_index = display_index
        self.model_name = model_name
        self.apps = apps or django_apps

        try:
            self.name, self.display_name = row
        except ValueError as e:
            raise ListModelMakerError(e) from e
        except TypeError as e:
            if "Row" not in str(e):
                raise
            self.name = row.name
            self.display_name = row.display_name
            self.extra_value = row.extra
            self.custom_name = row.custom_name

    def create_or_update(self):
        opts = dict(
            display_index=self.display_index,
            display_name=self.display_name,
            extra_value=self.extra_value or "",
        )
        if "custom_name" in [f.name for f in self.model_cls._meta.get_fields()]:
            opts.update(custom_name=self.custom_name)
        try:
            obj = self.model_cls.objects.get(name=self.name)
        except ObjectDoesNotExist:
            obj = self.model_cls.objects.create(name=self.name, **opts)
        else:
            for k, v in opts.items():
                setattr(obj, k, v)
            obj.save()
        return obj

    @property
    def model_cls(self):
        return self.apps.get_model(self.model_name)



# Generated by Django 5.1.2 on 2024-11-17 19:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0035_container_max_per_subject_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalstock",
            name="confirmed_at_site",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="historicalstock",
            name="confirmed_at_site_by",
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AddField(
            model_name="historicalstock",
            name="confirmed_at_site_datetime",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="stock",
            name="confirmed_at_site",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="stock",
            name="confirmed_at_site_by",
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AddField(
            model_name="stock",
            name="confirmed_at_site_datetime",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="confirmed_by",
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name="stock",
            name="confirmed_by",
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
    ]



from clinicedc_constants import OTHER, UNKNOWN

list_data = {
    "edc_dx_review.diagnosislocations": [
        ("hospital", "Hospital"),
        ("gov_clinic", "Government clinic"),
        ("private_clinic", "Private clinic"),
        ("private_doctor", "Private doctor"),
        ("study_clinic", "Study clinic"),
        (UNKNOWN, "Don't recall"),
        (OTHER, "Other, specify"),
    ],
    "edc_dx_review.reasonsfortesting": [
        ("patient_request", "Patient was well and made a request"),
        ("patient_complication", "Patient had a clinical complication"),
        ("signs_symptoms", "Patient had suggestive signs and symptoms"),
        (OTHER, "Other reason (specify below)"),
    ],
}



# Generated by Django 4.2.7 on 2023-12-04 22:19

import uuid

from django.db import migrations, models

import edc_pharmacy.models.prescription
import edc_sites.models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_pharmacy",
            "0020_alter_box_device_created_alter_box_device_modified_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="box",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Box",
                "verbose_name_plural": "Boxes",
            },
        ),
        migrations.AlterModelOptions(
            name="container",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Container",
                "verbose_name_plural": "Containers",
            },
        ),
        migrations.AlterModelOptions(
            name="containertype",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                )
            },
        ),
        migrations.AlterModelOptions(
            name="dispensinghistory",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Dispensing history",
                "verbose_name_plural": "Dispensing history",
            },
        ),
        migrations.AlterModelOptions(
            name="dosageguideline",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Dosage Guideline",
                "verbose_name_plural": "Dosage Guidelines",
            },
        ),
        migrations.AlterModelOptions(
            name="formulation",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Formulation",
                "verbose_name_plural": "Formulations",
            },
        ),
        migrations.AlterModelOptions(
            name="formulationtype",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "FormulationType",
                "verbose_name_plural": "FormulationTypes",
            },
        ),
        migrations.AlterModelOptions(
            name="frequencyunits",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Frequency units",
                "verbose_name_plural": "Frequency units",
            },
        ),
        migrations.AlterModelOptions(
            name="genericcontainer",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Item",
                "verbose_name_plural": "Items",
            },
        ),
        migrations.AlterModelOptions(
            name="labels",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Label",
                "verbose_name_plural": "Labels",
            },
        ),
        migrations.AlterModelOptions(
            name="location",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Location",
                "verbose_name_plural": "Locations",
            },
        ),
        migrations.AlterModelOptions(
            name="medication",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication",
                "verbose_name_plural": "Medications",
            },
        ),
        migrations.AlterModelOptions(
            name="medicationlot",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication lot",
                "verbose_name_plural": "Medication lots",
            },
        ),
        migrations.AlterModelOptions(
            name="order",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication order",
                "verbose_name_plural": "Medication orders",
            },
        ),
        migrations.AlterModelOptions(
            name="pillbottle",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Pill Bottle",
                "verbose_name_plural": "Pill Bottles",
            },
        ),
        migrations.AlterModelOptions(
            name="product",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication product",
                "verbose_name_plural": "Medication products",
            },
        ),
        migrations.AlterModelOptions(
            name="returnhistory",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Return history",
                "verbose_name_plural": "Return history",
            },
        ),
        migrations.AlterModelOptions(
            name="room",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Room",
                "verbose_name_plural": "Rooms",
            },
        ),
        migrations.AlterModelOptions(
            name="route",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Route",
                "verbose_name_plural": "Routes",
            },
        ),
        migrations.AlterModelOptions(
            name="rx",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Prescription",
                "verbose_name_plural": "Prescriptions",
            },
        ),
        migrations.AlterModelOptions(
            name="rxrefill",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "RX refill",
                "verbose_name_plural": "RX refills",
            },
        ),
        migrations.AlterModelOptions(
            name="shelf",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Shelf",
                "verbose_name_plural": "Shelves",
            },
        ),
        migrations.AlterModelOptions(
            name="stock",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication stock",
                "verbose_name_plural": "Medication stock",
            },
        ),
        migrations.AlterModelOptions(
            name="stockcreatelabels",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication stock: Create labels",
                "verbose_name_plural": "Medication stock: Create labels",
            },
        ),
        migrations.AlterModelOptions(
            name="stockreceiving",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication stock: Receiving",
                "verbose_name_plural": "Medication stock: Receiving",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectpillbottle",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Subject Pill Bottle",
                "verbose_name_plural": "Subject Pill Bottles",
            },
        ),
        migrations.AlterModelOptions(
            name="units",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Units",
                "verbose_name_plural": "Units",
            },
        ),
        migrations.AlterModelOptions(
            name="unittype",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                )
            },
        ),
        migrations.AlterModelManagers(
            name="rx",
            managers=[
                ("objects", edc_pharmacy.models.prescription.rx.Manager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.RemoveIndex(
            model_name="container",
            name="edc_pharmac_id_b68aba_idx",
        ),
        migrations.RemoveIndex(
            model_name="containertype",
            name="edc_pharmac_id_554046_idx",
        ),
        migrations.RemoveIndex(
            model_name="formulationtype",
            name="edc_pharmac_id_ff22c9_idx",
        ),
        migrations.RemoveIndex(
            model_name="frequencyunits",
            name="edc_pharmac_id_840f3f_idx",
        ),
        migrations.RemoveIndex(
            model_name="route",
            name="edc_pharmac_id_b35112_idx",
        ),
        migrations.RemoveIndex(
            model_name="units",
            name="edc_pharmac_id_93aa79_idx",
        ),
        migrations.RemoveIndex(
            model_name="unittype",
            name="edc_pharmac_id_8faa96_idx",
        ),
        migrations.AlterUniqueTogether(
            name="dispensinghistory",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="dosageguideline",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="formulation",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="medication",
            unique_together=set(),
        ),
        migrations.AddField(
            model_name="box",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="box",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="dispensinghistory",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="dispensinghistory",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="dosageguideline",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="dosageguideline",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="formulation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="formulation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="genericcontainer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="genericcontainer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldispensinghistory",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldispensinghistory",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldosageguideline",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldosageguideline",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalformulation",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalformulation",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicallabels",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicallabels",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalmedication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalmedication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalmedicationlot",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalmedicationlot",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalorder",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalorder",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalproduct",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalproduct",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalreturnhistory",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalreturnhistory",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalrx",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalrx",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalrxrefill",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalrxrefill",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalstock",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalstock",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalstockcreatelabels",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalstockcreatelabels",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalstockreceiving",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalstockreceiving",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="labels",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="labels",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="location",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="location",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="medication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="medication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="medicationlot",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="medicationlot",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="pillbottle",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="pillbottle",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="product",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="product",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="returnhistory",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="returnhistory",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="room",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="room",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="rx",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="rx",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="rxrefill",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="rxrefill",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="shelf",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="shelf",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="stock",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="stock",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="stockcreatelabels",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="stockcreatelabels",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="stockreceiving",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="stockreceiving",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjectpillbottle",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjectpillbottle",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="containertype",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="containertype",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="containertype",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="formulationtype",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="formulationtype",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="formulationtype",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="frequencyunits",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="frequencyunits",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="frequencyunits",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="rx_identifier",
            field=models.CharField(db_index=True, default=uuid.uuid4, max_length=36),
        ),
        migrations.AlterField(
            model_name="route",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="route",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="route",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="rx_identifier",
            field=models.CharField(default=uuid.uuid4, max_length=36, unique=True),
        ),
        migrations.AlterField(
            model_name="units",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="units",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="units",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="unittype",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="unittype",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="unittype",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AddIndex(
            model_name="box",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_1b5950_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="box",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_d39cd1_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="container",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_0172ab_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="containertype",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_54ef1a_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="dispensinghistory",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_fb4eea_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="dispensinghistory",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_a6dfb9_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="dosageguideline",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_19a9bf_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="dosageguideline",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_8fded1_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="formulation",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_b09151_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="formulation",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_c3278d_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="formulationtype",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_ae9253_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="frequencyunits",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_74a9fb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="genericcontainer",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_8a380a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="genericcontainer",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_985106_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="labels",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_a6e280_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="labels",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_bd2b4f_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="location",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_c4a1aa_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="location",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_1838af_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="medication",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_9ca5c6_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="medication",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_471fe8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="medicationlot",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_ac85b9_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="medicationlot",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_c69b56_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="order",
            index=models.Index(
                fields=["subject_identifier"], name="edc_pharmac_subject_36f4ea_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="order",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_dd4593_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="order",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_c19a6f_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="pillbottle",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_480869_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pillbottle",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_71fb9d_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_f3e4c4_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_3e12e8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="returnhistory",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_698780_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="returnhistory",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_c6202e_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="room",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_1f05db_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="room",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_687474_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="route",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_7e4edc_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="rx",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_986021_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="rx",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_b9ac53_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="rxrefill",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_07388c_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="rxrefill",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_451240_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="shelf",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_b28e36_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="shelf",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_392c48_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="stock",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_9836a2_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="stock",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_be4ea3_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="stockcreatelabels",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_3dc094_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="stockcreatelabels",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_434ace_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="stockreceiving",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_b91a43_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="stockreceiving",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_74be33_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="subjectpillbottle",
            index=models.Index(
                fields=["modified", "created"], name="edc_pharmac_modifie_c94c44_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="subjectpillbottle",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_pharmac_user_mo_230575_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="units",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_f1b67e_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="unittype",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_pharmac_display_60c19d_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="dispensinghistory",
            constraint=models.UniqueConstraint(
                fields=("rx_refill", "dispensed_datetime"),
                name="edc_pharmacy_dispensinghistory_rx_uniq",
            ),
        ),
        migrations.AddConstraint(
            model_name="dosageguideline",
            constraint=models.UniqueConstraint(
                fields=("medication", "dose", "dose_units", "dose_per_kg"),
                name="edc_pharmacy_dosageguideline_med_dose_uniq",
            ),
        ),
        migrations.AddConstraint(
            model_name="formulation",
            constraint=models.UniqueConstraint(
                fields=("medication", "strength", "units", "formulation_type"),
                name="edc_pharmacy_formulation_med_stren_uniq",
            ),
        ),
        migrations.AddConstraint(
            model_name="medication",
            constraint=models.UniqueConstraint(
                fields=("name", "display_name"),
                name="edc_pharmacy_medication_name_uniq",
            ),
        ),
        migrations.AddConstraint(
            model_name="returnhistory",
            constraint=models.UniqueConstraint(
                fields=("rx_refill", "return_datetime"),
                name="edc_pharmacy_returnhistory_rx_uniq",
            ),
        ),
    ]



from __future__ import annotations

from dateutil.relativedelta import FR, MO, SA, SU, TH, TU, WE

from .constants import (
    FIVE_DAY_CLINIC,
    FOUR_DAY_CLINIC,
    SEVEN_DAY_CLINIC,
    SIX_DAY_CLINIC,
    TU_WE_TH_CLINIC,
)

default_definitions: dict[str, dict[str, list | bool]] = {
    SEVEN_DAY_CLINIC: dict(
        days=[MO, TU, WE, TH, FR, SA, SU], slots=[100, 100, 100, 100, 100, 100, 100]
    ),
    SIX_DAY_CLINIC: dict(days=[MO, TU, WE, TH, FR, SA], slots=[100, 100, 100, 100, 100, 100]),
    FIVE_DAY_CLINIC: dict(days=[MO, TU, WE, TH, FR], slots=[100, 100, 100, 100, 100]),
    FOUR_DAY_CLINIC: dict(days=[MO, TU, WE, TH], slots=[100, 100, 100, 100]),
    TU_WE_TH_CLINIC: dict(
        days=[TU, WE, TH],
        slots=[100, 100, 100],
        best_effort_available_datetime=True,
    ),
}



from django.apps import AppConfig as DjangoAppConfig
from django.core.checks.registry import register

from .system_checks import middleware_check


class AppConfig(DjangoAppConfig):
    name = "edc_review_dashboard"

    def ready(self):
        register(middleware_check)



# Generated by Django 4.2.11 on 2024-06-17 21:40

import _socket
import django.db.models.deletion
import django.db.models.manager
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
from django.db import migrations, models

import edc_sites.managers
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.CreateModel(
            name="ReportNote",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("report_model", models.CharField(max_length=150)),
                (
                    "report_datetime",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                ("note", models.TextField(null=True)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("New", "New"),
                            ("open", "Open"),
                            ("resolved", "Resolved"),
                            ("closed", "Closed"),
                        ],
                        default="New",
                        max_length=25,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Note",
                "verbose_name_plural": "Notes",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["modified", "created"],
                        name="edc_qarepor_modifie_fd8974_idx",
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="edc_qarepor_user_mo_9875e1_idx",
                    ),
                ],
            },
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AddConstraint(
            model_name="reportnote",
            constraint=models.UniqueConstraint(
                fields=("report_model", "subject_identifier"),
                name="edc_qareports_reportnote_name_subj_uniq",
            ),
        ),
    ]



from .constants import DEFERRED_VISIT, LOST_VISIT, MISSED_VISIT, SCHEDULED, UNSCHEDULED


def constants(request) -> dict:
    return dict(
        DEFERRED_VISIT=DEFERRED_VISIT,
        LOST_VISIT=LOST_VISIT,
        MISSED_VISIT=MISSED_VISIT,
        SCHEDULED=SCHEDULED,
        UNSCHEDULED=UNSCHEDULED,
    )



import re

import pandas as pd
from bs4 import BeautifulSoup
from django.apps import apps as django_apps


def strip_html(text: str) -> str:
    if pd.isna(text):
        return text
    if bool(re.search(r"<[^>]+>", text)):
        return BeautifulSoup(text, "html.parser").get_text()
    return text


# def get_stata_labels_from_model(df: pd.DataFrame, model: str, suffix: str) -> dict[str:str]:
#     """Generate STATA labels"""
#     labels = {}
#     _, model_name = model.split(".")
#     model_cls = django_apps.get_model(model)
#     for fld in model_cls._meta.get_fields():
#         if f"{fld.name}_{suffix}" in df.columns:
#             labels.update({f"{fld.name}_{suffix}": strip_html(str(fld.verbose_name)[:80])})
#     return labels


def get_stata_labels_from_model(
    df: pd.DataFrame, model: str, suffix: str | None = None
) -> dict[str:str]:
    """Generate STATA labels"""
    labels = {}
    _, model_name = model.split(".")
    model_cls = django_apps.get_model(model)
    for fld in model_cls._meta.get_fields():
        if suffix:
            if f"{fld.name}_{suffix}" in df.columns:
                labels.update({f"{fld.name}_{suffix}": strip_html(str(fld.verbose_name)[:80])})
        elif f"{fld.name}_{suffix}" in df.columns:
            try:
                labels.update({fld.name: strip_html(str(fld.verbose_name)[:80])})
            except AttributeError:
                pass
    return labels



from django.db import models
from django_pylabels.models import LabelSpecification

from edc_model.models import BaseUuidModel
from edc_pylabels.site_label_configs import site_label_configs


class LabelConfigurationError(Exception):
    pass


class LabelConfiguration(BaseUuidModel):

    name = models.CharField(
        verbose_name="System config name",
        max_length=50,
        unique=True,
        help_text="Name of configuration registered with site_label_config.",
    )

    label_specification = models.ForeignKey(LabelSpecification, on_delete=models.PROTECT)

    requires_allocation = models.BooleanField(
        verbose_name="Configuration is for a subject label", default=False
    )

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.name not in site_label_configs.all():
            raise LabelConfigurationError(
                "Label configuration not registered with site_label_config."
            )
        super().save(*args, **kwargs)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Label configuration"
        verbose_name_plural = "Label configurations"



from .box_view_mixin import BoxViewMixin
from .clean_search_term_mixin import CleanSearchTermViewMixin
from .form_action_view_mixin import FormActionViewMixin
from .manifest_view_mixin import ManifestViewMixin
from .process_requisition_view_mixin import ProcessRequisitionViewMixin



from zoneinfo import ZoneInfo

from django import forms
from django.apps import apps as django_apps
from django.conf import settings

from edc_crf.crf_form_validator import CrfFormValidator
from edc_utils.text import convert_php_dateformat

from ..base_form_validator import INVALID_ERROR


class StudyDayFormValidatorMixin:
    def validate_study_day_with_datetime(
        self: CrfFormValidator,
        subject_identifier=None,
        study_day=None,
        compare_date=None,
        study_day_field=None,
    ):
        """Raises an exception if study day does not match
        calculation against UTC.

        Note: study-day is 1-based.
        """
        if study_day is not None and compare_date is not None:
            try:
                compare_date = compare_date.date()
            except AttributeError:
                pass
            if not subject_identifier or self.subject_identifier:
                raise ValueError(f"Subject identifier cannot be None. See {self!r}")
            registered_subject_model_cls = django_apps.get_model(
                "edc_registration.registeredsubject"
            )
            randomization_datetime = registered_subject_model_cls.objects.get(
                subject_identifier=subject_identifier
            ).randomization_datetime
            days_on_study = (compare_date - randomization_datetime.date()).days
            if study_day - 1 != days_on_study:
                randomization_datetime.astimezone(ZoneInfo(settings.TIME_ZONE))
                formatted_date = randomization_datetime.astimezone(
                    ZoneInfo(settings.TIME_ZONE)
                ).strftime(convert_php_dateformat(settings.DATETIME_FORMAT))
                message = {
                    study_day_field: (
                        f"Invalid. Expected {days_on_study + 1}. "
                        f"Subject was registered on {formatted_date}"
                    )
                }
                self._errors.update(message)
                self._error_codes.append(INVALID_ERROR)
                raise forms.ValidationError(message, code=INVALID_ERROR)



from __future__ import annotations

from functools import cache
from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.conf import settings
from django.contrib import admin

if TYPE_CHECKING:
    from django.contrib.admin import ModelAdmin
    from django.db.models import QuerySet
    from django.forms import ModelForm

    from edc_metadata.model_mixins.creates import CreatesMetadataModelMixin
    from edc_model.models import BaseUuidModel
    from edc_sites.model_mixins import SiteModelMixin
    from edc_visit_tracking.model_mixins import VisitModelMixin as Base

    from .models import Issue

    class RelatedVisitModel(SiteModelMixin, CreatesMetadataModelMixin, Base, BaseUuidModel):
        pass


__all__ = [
    "get_form_runner_issues",
    "get_edc_form_runners_enabled",
    "get_issue_model_cls",
    # "get_modelforms_from_admin_sites",
    "get_modeladmins_from_admin_sites",
    "get_modeladmin_cls",
]


def get_edc_form_runners_enabled() -> bool:
    return getattr(settings, "EDC_FORM_RUNNERS_ENABLED", False)


def get_issue_model_cls() -> Issue:
    return django_apps.get_model("edc_form_runners.issue")


@cache
def get_modeladmins_from_admin_sites() -> dict[str, type[ModelAdmin]]:
    registry = {}
    for admin_site in admin.sites.all_sites:
        registry.update(**get_modeladmins_from_admin_site(admin_site))
    return registry


@cache
def get_modelforms_from_admin_site(admin_site) -> dict[str, type[ModelForm]]:
    registry = {}
    for admin_class in admin_site._registry.values():
        registry.update({admin_class.model._meta.label_lower: admin_class.form})
    return registry


@cache
def get_modeladmins_from_admin_site(admin_site) -> dict[str, type[ModelAdmin]]:
    registry = {}
    for admin_class in admin_site._registry.values():
        registry.update({admin_class.model._meta.label_lower: admin_class})
    return registry


def get_modeladmin_cls(model_name: str) -> type[ModelAdmin]:
    return get_modeladmins_from_admin_sites().get(model_name)


def get_form_runner_issues(
    model_name: str, related_visit: RelatedVisitModel, panel_name: str | None = None
) -> QuerySet[Issue] | None:
    return get_issue_model_cls().objects.filter(
        subject_identifier=related_visit.subject_identifier,
        label_lower=model_name,
        visit_code=related_visit.visit_code,
        visit_code_sequence=related_visit.visit_code_sequence,
        visit_schedule_name=related_visit.visit_schedule_name,
        schedule_name=related_visit.schedule_name,
        panel_name=panel_name,
    )


# @cache
# def get_modelforms_from_admin_sites() -> dict[str, Type[ModelForm]]:
#     registry = {}
#     for admin_site in admin.sites.all_sites:
#         registry.update(**get_modelforms_from_admin_site(admin_site))
#     return registry

# def get_modelform_cls(model_name: str) -> Type[ModelForm]:
#     return get_modelforms_from_admin_sites().get(model_name)



from uuid import uuid4

from clinicedc_constants import YES
from django.core.validators import RegexValidator
from django.db import models
from django.db.models import PROTECT

from edc_constants.choices import YES_NO


class StudyMedicationRefillModelMixin(models.Model):
    refill = models.CharField(
        verbose_name="Will the subject receive study medication for this visit",
        max_length=15,
        default=YES,
        choices=YES_NO,
        help_text="If NO, set refill_start_datetime equal to the refill_end_datetime",
    )

    refill_start_datetime = models.DateTimeField()

    refill_end_datetime = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Leave blank to auto-calculate if refilling to the next scheduled visit",
    )

    refill_identifier = models.CharField(max_length=36, default=uuid4, editable=False)

    dosage_guideline = models.ForeignKey(
        "edc_pharmacy.DosageGuideline", on_delete=PROTECT, null=True, blank=False
    )

    formulation = models.ForeignKey(
        "edc_pharmacy.Formulation", on_delete=PROTECT, null=True, blank=False
    )

    roundup_divisible_by = models.IntegerField(default=1)

    refill_to_next_visit = models.CharField(
        verbose_name="Refill to the next scheduled visit",
        max_length=25,
        choices=YES_NO,
        default=YES,
        help_text="If YES, leave refill end date blank to auto-calculate",
    )

    number_of_days = models.IntegerField(
        null=True,
        blank=True,
        help_text="Leave blank to auto-calculate relative to the next scheduled appointment",
    )

    stock_codes = models.TextField(
        max_length=30,
        default="",
        blank=True,
        validators=[
            RegexValidator(
                # regex="^([A-Z0-9]{6})(,[A-Z0-9]{6})*$",
                regex="^([A-Z0-9]{6})(\r\n[A-Z0-9]{6})*$",
                message="Enter one or more valid codes, one code per line",
            )
        ],
        help_text="Enter the medication bottle barcode or barcodes. Type one code per line",
    )

    special_instructions = models.TextField(blank=True, default="")

    class Meta:
        verbose_name = "Study Medication"
        verbose_name_plural = "Study Medication"
        abstract = True



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-12-04 22:27
from __future__ import unicode_literals

import django_audit_fields.fields.uuid_auto_field
import django_revision.revision_field
from django.db import migrations, models

import edc_model_fields.fields.hostname_modification_field
import edc_model_fields.fields.userfield
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [("edc_identifier", "0006_auto_20161127_2226")]

    operations = [
        migrations.CreateModel(
            name="IdentifierModel",
            fields=[
                (
                    "created",
                    models.DateTimeField(default=django.utils.timezone.now, editable=False),
                ),
                (
                    "modified",
                    models.DateTimeField(default=django.utils.timezone.now, editable=False),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        editable=False,
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        editable=False,
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        default="mac2-2.local",
                        editable=False,
                        help_text="System field. (modified on create only)",
                        max_length=50,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        editable=False,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=50)),
                ("sequence_number", models.IntegerField()),
                ("identifier", models.CharField(max_length=50, unique=True)),
                ("device_id", models.IntegerField(default=99)),
                ("protocol_number", models.CharField(max_length=3)),
                ("model", models.CharField(max_length=50, null=True)),
                ("subject_type", models.CharField(max_length=25, null=True)),
                ("study_site", models.CharField(max_length=25)),
            ],
            options={"ordering": ["sequence_number"]},
        ),
        migrations.AlterUniqueTogether(
            name="identifiermodel", unique_together=set([("name", "identifier")])
        ),
    ]



navbars = [
    "edc_navbar.nav_administration",
    "edc_navbar.nav_home",
    "edc_navbar.nav_export_section",
    "edc_navbar.nav_logout",
    "edc_navbar.nav_pharmacy_section",
    "edc_navbar.nav_public",
    "edc_navbar.nav_enrolment_section",
    "edc_navbar.nav_subject_section",
]

custom_codename_tuples = []
for codename in navbars:
    custom_codename_tuples.append((codename, f"Can access {codename.split('.')[1]}"))



from __future__ import annotations

from typing import Any

from django.conf import settings


class DiagnosisLabelError(Exception):
    pass


def get_diagnosis_labels() -> dict:
    diagnosis_labels = getattr(
        settings,
        "EDC_DX_LABELS",
        dict(hiv="HIV", dm="Diabetes", htn="Hypertension", chol="High Cholesterol"),
    )
    return {k.lower(): v for k, v in diagnosis_labels.items()}


def get_diagnosis_labels_prefixes() -> list[str]:
    return [k for k in get_diagnosis_labels()]


def raise_on_unknown_diagnosis_labels(obj: Any, fld_suffix: str, fld_value: Any) -> None:
    """Raises an exception if a diagnosis field has a response
    but is not an expected condition.

    See also EDC_DX_LABELS.
    """
    labels = [
        fld.name.split(fld_suffix)[0]
        for fld in obj._meta.get_fields()
        if fld.name.endswith(fld_suffix)
        and fld.name.split(fld_suffix)[0] not in get_diagnosis_labels_prefixes()
        and getattr(obj, fld.name) == fld_value
    ]
    if labels:
        raise DiagnosisLabelError(
            "Diagnosis prefix not expected. See settings.EDC_DX_LABELS. "
            f"Expected one of {get_diagnosis_labels_prefixes()}. Got {labels}."
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps

if TYPE_CHECKING:
    from django.contrib.sites.models import Site


class SiteFormValidatorMixin:

    @property
    def site_model_cls(self) -> Site:
        return django_apps.get_model("sites.site")

    @property
    def site(self) -> Site:
        return (
            self.cleaned_data.get("site")
            or getattr(self.instance, "site", None)
            or self.site_model_cls.objects.get_current()
        )



# Generated by Django 4.2.1 on 2023-07-05 02:16

import django.db.models.manager
from django.db import migrations

import edc_sites.models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_data_manager", "0027_alter_dataquery_dm_user"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="dataquery",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Data Query",
                "verbose_name_plural": "Data Queries",
            },
        ),
        migrations.AlterModelOptions(
            name="queryrule",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("title",),
                "verbose_name": "Query Rule",
                "verbose_name_plural": "Query Rules",
            },
        ),
        migrations.AlterModelManagers(
            name="dataquery",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
    ]



# Generated by Django 4.2.7 on 2023-11-28 19:39

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.CreateModel(
            name="Issue",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("session_id", models.UUIDField(null=True)),
                ("session_datetime", models.DateTimeField(null=True)),
                ("label_lower", models.CharField(max_length=150)),
                ("verbose_name", models.CharField(max_length=150)),
                ("subject_identifier", models.CharField(max_length=150)),
                ("visit_code", models.CharField(max_length=150)),
                ("visit_code_sequence", models.CharField(max_length=150)),
                ("field_name", models.CharField(max_length=150)),
                ("raw_message", models.TextField(null=True)),
                ("message", models.TextField(null=True)),
                ("short_message", models.CharField(max_length=250, null=True)),
                ("response", models.CharField(max_length=250, null=True)),
                ("src_id", models.UUIDField(null=True)),
                ("src_revision", models.CharField(max_length=150, null=True)),
                ("src_report_datetime", models.DateTimeField(null=True)),
                ("src_modified_datetime", models.DateTimeField(null=True)),
                ("src_user_modified", models.CharField(max_length=150, null=True)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("New", "New"),
                            ("in_progress", "In progress"),
                            ("done", "Done"),
                        ],
                        default="New",
                        max_length=15,
                    ),
                ),
                ("extra_formfields", models.TextField(null=True)),
                ("ignore_formfields", models.TextField(null=True)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Issue",
                "verbose_name_plural": "Issues",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["label_lower", "field_name", "short_message"],
                        name="edc_form_ru_label_l_ce3dda_idx",
                    ),
                    models.Index(
                        fields=[
                            "subject_identifier",
                            "visit_code",
                            "visit_code_sequence",
                            "short_message",
                        ],
                        name="edc_form_ru_subject_10bb16_idx",
                    ),
                ],
            },
        ),
        migrations.AddConstraint(
            model_name="issue",
            constraint=models.UniqueConstraint(
                fields=(
                    "label_lower",
                    "subject_identifier",
                    "field_name",
                    "visit_code",
                    "visit_code_sequence",
                ),
                name="unique_label_lower_subject_identifier_etc",
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-10 13:36

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_export", "0009_auto_20210510_2036"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="exportdata",
            options={
                "permissions": [("display_export_admin_action", "Display export action")]
            },
        ),
        migrations.AlterModelOptions(
            name="importdata",
            options={
                "permissions": [("display_import_admin_action", "Display import action")]
            },
        ),
    ]



from django import forms

from ..modelform_mixins import DeathReportModelFormMixin
from ..utils import get_ae_model


class DeathReportForm(DeathReportModelFormMixin, forms.ModelForm):
    class Meta(DeathReportModelFormMixin.Meta):
        model = get_ae_model("deathreport")
        fields = "__all__"



from django.apps import AppConfig as DjangoAppConfig


class AppConfig(DjangoAppConfig):
    name = "edc_ltfu"
    verbose_name = "Edc Loss to Follow up"
    has_exportable_data = True
    include_in_administration_section = True



from django import forms

from ...models import Supplier


class SupplierForm(forms.ModelForm):
    class Meta:
        model = Supplier
        fields = "__all__"
        help_text = {"supplier_identifier": "(read-only)"}  # noqa: RUF012
        widgets = {  # noqa: RUF012
            "supplier_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from django import forms

from ...models import Product, Stock


class ProductForm(forms.ModelForm):
    def clean(self):
        cleaned_data = super().clean()
        if (
            getattr(self.instance, "id", None)
            and Stock.objects.filter(product=self.instance).exists()
        ):
            raise forms.ValidationError("Product is in use and cannot be changed.")
        return cleaned_data

    class Meta:
        model = Product
        fields = "__all__"
        help_text = {"product_identifier": "(read-only)"}  # noqa: RUF012
        widgets = {  # noqa: RUF012
            "product_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_identifier", "0007_alter_identifiermodel_device_created_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="identifiermodel",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterUniqueTogether(
            name="identifiermodel",
            unique_together=set(),
        ),
        migrations.AddField(
            model_name="identifiermodel",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="identifiermodel",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddIndex(
            model_name="identifiermodel",
            index=models.Index(
                fields=["modified", "created"], name="edc_identif_modifie_1b75ca_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="identifiermodel",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_identif_user_mo_357b53_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="identifiermodel",
            constraint=models.UniqueConstraint(
                fields=("name", "identifier"),
                name="edc_identifier_identifiermodel_name_uniq",
            ),
        ),
    ]



from typing import Any

from django.apps import apps as django_apps
from django.views.generic.base import TemplateView

from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from .view_mixins import EdcDeviceViewMixin


class HomeView(EdcViewMixin, NavbarViewMixin, EdcDeviceViewMixin, TemplateView):
    template_name = "edc_device/home.html"
    navbar_name = "edc_device"
    navbar_selected_item = "device"

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        kwargs.update(
            {
                "project_name": (
                    f"{kwargs.get('project_name')}: "
                    f"{django_apps.get_app_config('edc_device').verbose_name}"
                )
            }
        )
        return super().get_context_data(**kwargs)

    def get_context_data_for_sites(self, **kwargs):
        return kwargs



import socket
import sys

from django.conf import settings
from django.core.management.base import BaseCommand
from django.core.management.color import color_style
from multisite.models import Alias
from multisite.utils import create_or_sync_canonical_from_all_sites

from edc_sites.site import sites as site_sites
from edc_sites.utils import add_or_update_django_sites

style = color_style()


class Command(BaseCommand):
    help = "Add / Update django Site model after changes to edc_sites"

    def add_arguments(self, parser):
        parser.add_argument(
            "--ping-hosts",
            default=False,
            action="store_true",
            dest="ping_hosts",
            help="Try to ping hosts",
        )

        parser.add_argument(
            "--suggest-hosts",
            default=False,
            action="store_true",
            dest="suggest_hosts",
            help="Suggest ALLOWED_HOSTS",
        )

    def handle(self, *args, **options) -> None:  # noqa: ARG002
        sys.stdout.write("\n\n")
        sys.stdout.write(" Edc Sites : Adding / Updating sites ...     \n")
        add_or_update_django_sites(verbose=True)
        sys.stdout.write("\n Multisite. \n")
        create_or_sync_canonical_from_all_sites(verbose=True)

        sys.stdout.write("    multisite.Alias \n")
        for obj in Alias.objects.all():
            sys.stdout.write(
                f"      - Site model: {obj.site.id}: {obj.domain} "
                f"is_canonical={obj.is_canonical}.\n"
            )
        arg = [arg for arg in sys.argv if arg.startswith("--settings")]
        sys.stdout.write(f"\n Settings: reading from {arg}")
        sys.stdout.write("\n\n Current value of settings.ALLOWED_HOSTS\n")
        sys.stdout.write(style.WARNING(f"\n    DEBUG = {settings.DEBUG}\n"))
        sys.stdout.write("    ALLOWED_HOSTS = [\n")
        for host in settings.ALLOWED_HOSTS:
            sys.stdout.write(f"      {host},\n")
        sys.stdout.write("    ]\n")
        sys.stdout.write("\n Suggested settings.ALLOWED_HOSTS\n\n")
        sys.stdout.write("    ALLOWED_HOSTS = [\n")
        for single_site in site_sites.all(aslist=True):
            sys.stdout.write(f"      {single_site.domain},\n")
        sys.stdout.write("    ]\n")
        if options.get("ping_hosts"):
            sys.stdout.write("\n Looking up hosts:\n")
            for single_site in site_sites.all(aslist=True):
                try:
                    host = socket.gethostbyname(single_site.domain)
                except socket.gaierror as e:
                    sys.stdout.write(
                        style.ERROR(f"  {single_site.domain}: <not found>. Got {e}\n")
                    )
                else:
                    sys.stdout.write(f"  {single_site.domain}: {host} {style.SUCCESS('OK')}\n")
        sys.stdout.write("Done     \n")



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Self

from django.db import models

from ...visit_sequence import VisitSequence, VisitSequenceError

if TYPE_CHECKING:
    from edc_appointment.models import Appointment
    from edc_visit_tracking.typing_stubs import RelatedVisitProtocol


class PreviousVisitError(Exception):
    pass


class PreviousVisitModelMixin(models.Model):
    """A model mixin to force the user to complete visit model
    instances in sequence.

    * Ensures the previous visit exists before allowing save()
      by raising PreviousVisitError.
    * If the visit is the first in the sequence, save() is allowed.
    """

    visit_sequence_cls: type[VisitSequence] = VisitSequence

    def save(self, *args, **kwargs) -> None:
        self.validate_visit_sequence()
        super().save(*args, **kwargs)

    def validate_visit_sequence(
        self: Appointment | RelatedVisitProtocol | Self,
    ) -> None:
        try:
            appointment = self.related_visit.appointment
        except AttributeError as e:
            if "related_visit" not in str(e):
                raise
            appointment = self.appointment
        visit_sequence = self.visit_sequence_cls(appointment=appointment)
        try:
            visit_sequence.enforce_sequence()
        except VisitSequenceError as e:
            raise PreviousVisitError(e) from e

    @property
    def previous_visit(self: Appointment | RelatedVisitProtocol | Self) -> Any:
        try:
            appointment = self.related_visit.appointment
        except AttributeError as e:
            if "related_visit" not in str(e):
                raise
            appointment = self.appointment
        visit_sequence = self.visit_sequence_cls(appointment=appointment)
        return visit_sequence.previous_visit

    class Meta:
        abstract = True



from __future__ import annotations

from datetime import datetime
from typing import Any

from django.conf import settings

from edc_form_validators import INVALID_ERROR
from edc_screening.form_validator_mixins import SubjectScreeningFormValidatorMixin
from edc_sites.site import sites
from edc_utils import AgeValueError, age
from edc_utils.date import to_local
from edc_utils.text import convert_php_dateformat

from ..site_consents import site_consents


class SubjectConsentFormValidatorMixin(SubjectScreeningFormValidatorMixin):
    """Form Validator mixin for the consent model."""

    def __init__(self: Any, **kwargs):
        super().__init__(**kwargs)
        self._consent_datetime = None

    def _clean(self) -> None:
        self.validate_demographics()
        super()._clean()

    def validate_demographics(self) -> None:
        self.validate_consent_datetime()
        self.validate_age()
        self.validate_gender()
        self.validate_identity()

    @property
    def gender(self):
        return self.cleaned_data.get("gender")

    @property
    def dob(self):
        return self.cleaned_data.get("dob")

    @property
    def guardian_name(self):
        return self.cleaned_data.get("guardian_name")

    @property
    def consent_model(self):
        return settings.SUBJECT_CONSENT_MODEL

    @property
    def consent_model_cls(self):
        cdef = site_consents.get_consent_definition(
            model=self.consent_model, site=sites.get(self.instance.site.id)
        )
        return cdef.model_cls

    @property
    def consent_datetime(self) -> datetime | None:
        if not self._consent_datetime:
            if "consent_datetime" in self.cleaned_data:
                if self.add_form and not self.cleaned_data.get("consent_datetime"):
                    self.raise_validation_error(
                        {"consent_datetime": "This field is required."}, INVALID_ERROR
                    )
                self._consent_datetime = self.cleaned_data.get("consent_datetime")
            else:
                self._consent_datetime = self.instance.consent_datetime
        return self._consent_datetime

    @property
    def screening_age_in_years(self) -> int:
        """Returns age in years calculated from dob relative to
        screening datetime"""
        try:
            rdelta = age(self.dob, self.subject_screening.report_datetime.date())
        except AgeValueError as e:
            self.raise_validation_error(str(e), INVALID_ERROR, exc=e)
        return rdelta.years

    def validate_age(self) -> None:
        """Validate age matches that on the screening form."""
        if self.dob and self.screening_age_in_years != self.subject_screening.age_in_years:
            self.raise_validation_error(
                {
                    "dob": "Age mismatch. The date of birth entered does "
                    f"not match the age at screening. "
                    f"Expected {self.subject_screening.age_in_years}. "
                    f"Got {self.screening_age_in_years}."
                },
                INVALID_ERROR,
            )

    def validate_gender(self) -> None:
        """Validate gender matches that on the screening form."""
        if self.gender != self.subject_screening.gender:
            self.raise_validation_error(
                {
                    "gender": "Gender mismatch. The gender entered does "
                    f"not match that reported at screening. "
                    f"Expected '{self.subject_screening.get_gender_display()}'. "
                    f"Got `{self.gender}`."
                },
                INVALID_ERROR,
            )

    def validate_consent_datetime(self) -> None:
        """Validate consent datetime with the eligibility datetime.

        Eligibility datetime must come first.

        Watchout for timezone, cleaned_data has local TZ.
        """
        if not self.subject_screening.eligibility_datetime:
            self.raise_validation_error(
                (
                    "Unable to determine the eligibility datetime from the "
                    f"screening form. See {self.subject_screening._meta.verbose_name}"
                    f"({self.subject_screening}). Got None."
                ),
                INVALID_ERROR,
            )
        if self.consent_datetime:
            if (
                self.consent_datetime - self.subject_screening.eligibility_datetime
            ).total_seconds() < 0:
                dt_str = to_local(self.subject_screening.eligibility_datetime).strftime(
                    convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
                )
                self.raise_validation_error(
                    {
                        "consent_datetime": (
                            f"Cannot be before the date and time eligibility "
                            f"was confirmed. Eligibility was confirmed at "
                            f"{dt_str}."
                        )
                    },
                    INVALID_ERROR,
                )

    def validate_identity(self: Any) -> None:
        pass



from .age import AgeTable
from .art import ArtTable
from .bmi import BmiTable
from .bp import BpTable
from .fasting import FastingTable
from .fbg import FbgTable
from .fbg_ogtt import FbgOgttTable
from .gender import GenderTable
from .hba1c import HbA1cTable
from .ogtt import OgttTable
from .waist import WaistCircumferenceTable



# Generated by Django 2.0 on 2018-01-02 11:58

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_facility", "0001_initial")]

    operations = [
        migrations.AlterModelOptions(
            name="holiday", options={"ordering": ["country", "local_date"]}
        ),
        migrations.AddField(
            model_name="holiday",
            name="country",
            field=models.CharField(default="botswana", max_length=25),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="holiday",
            name="local_date",
            field=models.DateField(default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="holiday",
            name="name",
            field=models.CharField(default="holiday", max_length=25),
            preserve_default=False,
        ),
        migrations.RemoveField(model_name="holiday", name="day"),
        migrations.AlterUniqueTogether(
            name="holiday", unique_together={("country", "local_date")}
        ),
    ]



from __future__ import annotations

from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from edc_utils import to_local


class HistoricalModelAdminMixin:
    change_list_note_url_name: str = ""

    @property
    def change_list_note(self):
        url = reverse(self.change_list_note_url_name)
        return format_html(
            "You are viewing the <B>transaction history</B> for this form. Click "
            '<A href="{}">here</A> to see the live data.',
            url,
        )

    @admin.display(description="History date", ordering="history_date")
    def formatted_history_date(self, obj):
        if obj.history_date:
            return to_local(obj.history_date)
        return None

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



import pandas as pd
from clinicedc_constants import FEMALE, MALE

from .row_statistics import RowStatistics


class RowStatisticsError(Exception):
    pass


class RowStatisticsFemale(RowStatistics):
    def __init__(
        self,
        df_numerator: pd.DataFrame = None,
        df_denominator: pd.DataFrame = None,
        **kwargs,
    ):
        df_numerator = df_numerator.loc[df_numerator["gender"] == FEMALE]
        super().__init__(
            df_numerator=df_numerator,
            df_denominator=df_denominator,
            **kwargs,
        )


class RowStatisticsMale(RowStatistics):
    def __init__(
        self,
        df_numerator: pd.DataFrame = None,
        df_denominator: pd.DataFrame = None,
        **kwargs,
    ):
        df_numerator = df_numerator.loc[df_numerator["gender"] == MALE]
        super().__init__(
            df_numerator=df_numerator,
            df_denominator=df_denominator,
            **kwargs,
        )


class RowStatisticsWithGender(RowStatistics):
    def __init__(
        self,
        columns: dict[str, tuple[str, int]] = None,
        df_all: pd.DataFrame = None,
        coltotal: float | int | None = None,
        **kwargs,
    ):
        """
        custom row for displaying with gender columns: F, M, All
        :param colname:
        :param df_numerator:
        :param df_denominator:
        :param df_all:
        :param columns: dict of {col: (style name, places)} where col
               is "F", "M" or "All"

        Note: the default df["gender"] is "M" or "F".
        """

        female_style, female_places = columns[FEMALE]
        male_style, male_places = columns[MALE]
        all_style, all_places = columns["All"]

        super().__init__(
            places=all_places,
            style=all_style,
            df_all=df_all,
            coltotal=coltotal,
            **kwargs,
        )

        self.m = RowStatisticsMale(
            places=male_places,
            style=male_style,
            coltotal=len(df_all[df_all["gender"] == MALE]),
            df_all=df_all,
            **kwargs,
        )
        self.f = RowStatisticsFemale(
            places=female_places,
            style=female_style,
            coltotal=len(df_all[df_all["gender"] == FEMALE]),
            df_all=df_all,
            **kwargs,
        )

    def values_list(self, style: str | None = None, places: int | None = None) -> list:
        values_list = super().values_list()
        return (
            list(self.formatted_cells().values())
            + self.f.values_list()
            + self.m.values_list()
            + values_list
        )

    def labels(self) -> list[str]:
        labels = super().labels()
        return (
            list(self.formatted_cells().keys())
            + [f"f{x}" for x in self.f.labels()]
            + [f"m{x}" for x in self.m.labels()]
            + labels
        )

    def row(self):
        return [self.formatted_cells()] + self.values_list()

    def formatted_cells(self) -> dict:
        formatted_cell = super().formatted_cell()
        return dict(
            F=self.f.formatted_cell(),
            M=self.m.formatted_cell(),
            All=formatted_cell,
        )



from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from ...view_mixins import ListboardFilterViewMixin, SearchFormViewMixin
from ..listboard_view import ListboardView


class SubjectListboardView(
    EdcViewMixin,
    NavbarViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    ListboardView,
):
    listboard_model = None

    listboard_template: str = "subject_listboard_template"
    listboard_url: str = "subject_listboard_url"
    listboard_panel_style: str = "success"
    listboard_fa_icon: str = "fas fa-user-circle fa-2x"
    listboard_view_permission_codename: str = "edc_subject_dashboard.view_subject_listboard"

    navbar_selected_item: str = "consented_subject"
    search_form_url: str = "subject_listboard_url"

    search_fields = (
        "user_created",
        "user_modified",
        "screening_identifier",
        "subject_identifier",
        "initials__exact",
        "identity__exact",
        "first_name__exact",
    )

    def get_listboard_model(self) -> str:
        return self.listboard_model



from django.apps import apps as django_apps
from django.conf import settings


def get_ltfu_model_name():
    model_name = getattr(settings, "EDC_LTFU_MODEL_NAME", None)
    if not model_name:
        raise ValueError("Model name may not be none. Got EDC_LTFU_MODEL_NAME=''")
    return getattr(settings, "EDC_LTFU_MODEL_NAME", None)


def get_ltfu_model_cls():
    return django_apps.get_model(get_ltfu_model_name())



from django.db import models

from edc_model import models as edc_models


class DxLocationModelMixin(models.Model):
    dx_location = models.ForeignKey(
        "edc_dx_review.diagnosislocations",
        verbose_name="Where was the diagnosis made?",
        on_delete=models.PROTECT,
        null=True,
        blank=False,
    )

    dx_location_other = edc_models.OtherCharField()

    class Meta:
        abstract = True



# Generated by Django 6.0 on 2026-01-19 17:11

from django.db import migrations
from sequences.models import Sequence
from tqdm import tqdm

from edc_pharmacy.models import Allocation, ConfirmationAtLocationItem, DispenseItem, Stock, \
    StockTransferItem, StorageBinItem


def update_sequences_for_model_renames(apps, schema_editor):
    Sequence.objects.filter(name="edc_pharmacy.confirmationatsiteitem").update(
        name="edc_pharmacy.confirmationatlocationitem"
    )
    Sequence.objects.filter(name="edc_pharmacy.confirmationatsite").update(
        name="edc_pharmacy.confirmationatlocation"
    )


def update_confirmation_at_location(apps, schema_editor):
    qs = ConfirmationAtLocationItem.objects.filter(stock_transfer_item__isnull=True)
    for obj in tqdm(qs, total=qs.count()):
        stock_obj = Stock.objects.get(code=obj.stock.code)
        if stock_obj.stocktransferitem_set.all().count() == 1:
            obj.stock_transfer_item = stock_obj.stocktransferitem_set.all()[0]
        obj.save()


def update_stock_codes_on_item_models(apps, schema_editor):
    qs = StockTransferItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()
    qs = StorageBinItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()


def update_subject_identifier(apps, schema_editor):
    Allocation.objects.filter(stock__isnull=True).delete()
    qs = Allocation.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.subject_identifier = obj.registered_subject.subject_identifier
        obj.code = obj.stock.code
        obj.save(update_fields=["subject_identifier", "code"])


def update_stock_boolean_fields(apps, schema_editor):

    print("\nUpdating boolean fields on the Stock model ...")
    print("  iterating over 1 / 4 models (StockTransferItem)...")
    qs = StockTransferItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()

    print("  iterating over 2 / 4 models (ConfirmationAtLocationItem)...")
    qs = ConfirmationAtLocationItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()

    print("  iterating over 3 / 4 models (StorageBinItem)...")
    qs = StorageBinItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()

    print("  iterating over 4 / 4 models (DispenseItem)...")
    qs = DispenseItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()
    print("Done.")


def update_dispense_item_stock_code(apps, schema_editor):
    qs = DispenseItem.objects.all()
    for obj in tqdm(qs, total=qs.count()):
        obj.save()


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0105_allocation_code_historicalallocation_code"),
    ]

    operations = [
        migrations.RunPython(update_sequences_for_model_renames),
        migrations.RunPython(update_confirmation_at_location),
        migrations.RunPython(update_stock_codes_on_item_models),
        migrations.RunPython(update_subject_identifier),
        migrations.RunPython(update_stock_boolean_fields),
        migrations.RunPython(update_dispense_item_stock_code),
    ]



# Generated by Django 5.2.8 on 2025-11-20 22:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_metadata", "0032_alter_crfmetadata_revision_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="crfmetadata",
            name="edc_metadat_site_id_d3b213_idx",
        ),
        migrations.RemoveIndex(
            model_name="crfmetadata",
            name="edc_metadat_subject_dd4400_idx",
        ),
        migrations.AddIndex(
            model_name="crfmetadata",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "visit_code",
                    "visit_code_sequence",
                    "show_order",
                ],
                name="edc_metadata_crfmetadata_a0idx",
            ),
        ),
        migrations.AddIndex(
            model_name="crfmetadata",
            index=models.Index(fields=["site"], name="edc_metadata_crfmetadata_a1idx"),
        ),
        migrations.AddIndex(
            model_name="crfmetadata",
            index=models.Index(
                fields=["entry_status"], name="edc_metadata_crfmetadata_a2idx"
            ),
        ),
        migrations.AddIndex(
            model_name="crfmetadata",
            index=models.Index(fields=["model"], name="edc_metadata_crfmetadata_a3idx"),
        ),
    ]



# Generated by Django 6.0 on 2026-01-19 16:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_pharmacy",
            "0101_rename_stored_at_site_historicalstock_stored_at_location_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="dispenseitem",
            name="code",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                max_length=15,
                verbose_name="Stock code",
            ),
        ),
        migrations.AddField(
            model_name="historicaldispenseitem",
            name="code",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                max_length=15,
                verbose_name="Stock code",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="stored_at_location",
            field=models.BooleanField(default=False, help_text="See storagebinitem."),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="item_count",
            field=models.PositiveIntegerField(
                help_text="Suggested item count", null=True
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="stored_at_location",
            field=models.BooleanField(default=False, help_text="See storagebinitem."),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="item_count",
            field=models.PositiveIntegerField(
                help_text="Suggested item count", null=True
            ),
        ),
    ]



from django import forms

from ...models import RepackRequest


class RepackRequestForm(forms.ModelForm):
    def clean(self):
        cleaned_data = super().clean()
        if cleaned_data.get("from_stock") and not getattr(
            cleaned_data.get("from_stock"), "confirmation", None
        ):
            raise forms.ValidationError(
                {
                    "from_stock": (
                        "Unconfirmed stock item. Only confirmed "
                        "stock items may be used to repack"
                    )
                }
            )
        if (
            cleaned_data.get("container")
            and cleaned_data.get("container") == cleaned_data.get("from_stock").container
        ):
            raise forms.ValidationError(
                {"container": "Stock is already packed in this container."}
            )
        if (
            cleaned_data.get("container")
            and cleaned_data.get("container").qty
            > cleaned_data.get("from_stock").container.qty
        ):
            raise forms.ValidationError({"container": "Cannot pack into larger container."})
        if (
            cleaned_data.get("requested_qty")
            and self.instance.processed_qty
            and cleaned_data.get("requested_qty") < self.instance.processed_qty
        ):
            raise forms.ValidationError(
                {"requested_qty": "Cannot be less than the number of containers processed"}
            )
        if (
            cleaned_data.get("requested_qty") * cleaned_data.get("container").qty
            > cleaned_data.get("from_stock").unit_qty
        ):
            needed_qty = cleaned_data.get("requested_qty") * cleaned_data.get("container").qty
            on_hand_qty = cleaned_data.get("from_stock").unit_qty
            raise forms.ValidationError(
                {
                    "requested_qty": (
                        "Insufficient unit quantity to repack from this stock item. "
                        f"Need {needed_qty} units but have only {on_hand_qty} units on hand"
                    )
                }
            )

        return cleaned_data

    class Meta:
        model = RepackRequest
        fields = "__all__"
        help_text = {  # noqa: RUF012
            "repack_identifier": "(read-only)",
        }
        widgets = {  # noqa: RUF012
            "repack_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from django.db import models
from django.utils.translation import gettext_lazy as _

from edc_model.models import BaseUuidModel


class Notification(BaseUuidModel):
    """A model that stores the notification types.

    Currently, show these for the user to select/subscribe to
    in a M2M in the edc_auth.UserProfile.

    For example:
        - a new model has been created
        - a death has occured
        - a grade 4 event has occured.
    """

    name = models.CharField(max_length=255, unique=True)

    display_name = models.CharField(max_length=255, unique=True)

    mailing_list_address = models.EmailField(_("Mailing list address"), blank=True, default="")

    enabled = models.BooleanField(default=True)

    def __str__(self):
        return f"{self.display_name}"

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Notification"
        verbose_name_plural = "Notifications"



from clinicedc_constants import CLOSED, HIGH_PRIORITY

from edc_action_item.action_with_notification import ActionWithNotification

from ..constants import (
    ADVERSE_EVENT_ADMIN_SITE,
    ADVERSE_EVENT_APP_LABEL,
    DEATH_REPORT_TMG_ACTION,
    DEATH_REPORT_TMG_SECOND_ACTION,
)


class DeathReportTmgSecondAction(ActionWithNotification):
    name = DEATH_REPORT_TMG_SECOND_ACTION
    display_name = "TMG Death Report (2nd) pending"
    notification_display_name = "TMG Death Report (2nd)"
    parent_action_names = (DEATH_REPORT_TMG_ACTION,)
    related_reference_fk_attr = "death_report"
    priority = HIGH_PRIORITY
    create_by_user = False
    color_style = "info"
    show_link_to_changelist = True
    singleton = True
    instructions = "This report is to be completed by the TMG only."

    reference_model = f"{ADVERSE_EVENT_APP_LABEL}.deathreporttmgsecond"
    related_reference_model = f"{ADVERSE_EVENT_APP_LABEL}.deathreport"
    admin_site_name = ADVERSE_EVENT_ADMIN_SITE

    def close_and_create_next(self):
        super().close_and_create_next()

    def reopen_action_item_on_change(self):
        """Do not reopen if report status is CLOSED."""
        return self.reference_obj.report_status != CLOSED

    def close_action_item_on_save(self):
        """Close if report status is CLOSED."""
        return self.reference_obj.report_status == CLOSED



from __future__ import annotations

from decimal import Decimal

from celery.states import PENDING
from clinicedc_constants import CANCEL, COMPLETE, PARTIAL
from django.db.models import Sum
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from edc_utils.celery import get_task_result, run_task_sync_or_async

from ..exceptions import InsufficientStockError
from ..model_mixins import StudyMedicationCrfModelMixin
from ..utils import (
    create_new_stock_on_receive,
    process_repack_request,
    update_previous_refill_end_datetime,
    update_stock_instance_qty,
)
from .stock import (
    Allocation,
    ConfirmationAtLocationItem,
    DispenseItem,
    OrderItem,
    Receive,
    ReceiveItem,
    RepackRequest,
    Stock,
    StockAdjustment,
    StockRequest,
    StockRequestItem,
    StockTransferItem,
    StorageBinItem,
)


@receiver(post_save, sender=Stock, dispatch_uid="update_stock_on_post_save")
def stock_on_post_save(sender, instance, raw, created, update_fields, **kwargs):
    """Update unit qty and other columns"""
    if not raw and not update_fields:
        instance = update_stock_instance_qty(instance)
        if instance.from_stock:
            # adjust the unit_qty_out, unit_qty_out on the source stock item
            # (from_stock). If insufficient, will bomb out here.
            instance.from_stock.save()


@receiver(
    post_save,
    sender=StockAdjustment,
    dispatch_uid="update_stock_adjustment_on_post_save",
)
def stock_adjustment_on_post_save(sender, instance, raw, created, update_fields, **kwargs):
    """Update unit qty"""
    if not raw and not update_fields:
        instance.stock.unit_qty_in = instance.unit_qty_in_new
        if instance.stock.unit_qty_out > instance.stock.unit_qty_in:
            raise InsufficientStockError(
                "Invalid adjustment. Expected a value greater than or equal to "
                f"{instance.stock.unit_qty_out}. See {instance}."
            )
        instance.stock.save(update_fields=["unit_qty_in"])


@receiver(post_save, sender=OrderItem, dispatch_uid="update_order_item_on_post_save")
def order_item_on_post_save(sender, instance, raw, created, update_fields, **kwargs):
    if not raw and not update_fields:
        # recalculate unit_qty
        unit_qty_ordered = instance.qty * instance.container.qty
        instance.unit_qty = unit_qty_ordered - (instance.unit_qty_received or Decimal(0))
        instance.save(update_fields=["unit_qty"])


@receiver(post_save, sender=Receive, dispatch_uid="receive_on_post_save")
def receive_on_post_save(sender, instance, raw, created, update_fields, **kwargs) -> None:
    if not raw and not update_fields:
        pass


@receiver(post_save, sender=ReceiveItem, dispatch_uid="update_receive_item_on_post_save")
def receive_item_on_post_save(sender, instance, raw, created, update_fields, **kwargs):
    if not raw and update_fields != ["added_to_stock"]:
        receive_items = ReceiveItem.objects.filter(receive=instance.receive)
        instance.receive.item_count = receive_items.count()
        instance.order_item.unit_qty_received = (
            instance.order_item.receiveitem_set.all().aggregate(unit_qty=Sum("unit_qty"))[
                "unit_qty"
            ]
        ) or Decimal(0.0)
        if instance.order_item.unit_qty_received == instance.order_item.unit_qty:
            instance.order_item.status = COMPLETE
        elif instance.order_item.unit_qty_received < instance.order_item.unit_qty:
            instance.order_item.status = PARTIAL
        instance.order_item.save()

        order = instance.receive.order
        unit_qty_received = OrderItem.objects.filter(order=order).aggregate(
            unit_qty_received=Sum("unit_qty_received")
        )["unit_qty_received"] or Decimal(0.0)
        unit_qty = OrderItem.objects.filter(order=order).aggregate(unit_qty=Sum("unit_qty"))[
            "unit_qty"
        ] or Decimal(0.0)
        if unit_qty_received == unit_qty:
            order.status = COMPLETE
            order.save()

        # add to stock
        create_new_stock_on_receive(receive_item_pk=instance.id)


@receiver(post_save, sender=StockRequest, dispatch_uid="stock_request_on_post_save")
def stock_request_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        if instance.cancel == CANCEL:
            if not Allocation.objects.filter(
                stock_request_item__stock_request=instance
            ).exists():
                instance.stockrequestitem_set.all().delete()
            else:
                instance.cancel = ""
                instance.save(update_fields=["cancel"])
        instance.item_count = instance.stockrequestitem_set.count()
        instance.save(update_fields=["item_count"])


@receiver(post_save, sender=StockRequestItem, dispatch_uid="stock_request_item_on_post_save")
def stock_request_item_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        instance.stock_request.item_count = StockRequestItem.objects.filter(
            stock_request=instance.stock_request
        ).count()
        instance.stock_request.save(update_fields=["item_count"])


@receiver(post_save, sender=RepackRequest, dispatch_uid="repack_request_on_post_save")
def repack_request_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        result = get_task_result(instance)
        if getattr(result, "state", "") == PENDING:
            pass
        else:
            task = run_task_sync_or_async(
                process_repack_request,
                repack_request_id=str(instance.id),
                username=instance.user_modified or instance.user_created,
            )
            instance.task_id = getattr(task, "id", None)
            instance.save(update_fields=["task_id"])


@receiver(
    post_save,
    sender=Allocation,
    dispatch_uid="allocation_on_post_save",
)
def allocation_on_post_save(sender, instance, raw, created, update_fields, **kwargs) -> None:
    instance.stock.subject_identifier = instance.registered_subject.subject_identifier
    instance.stock.save(update_fields=["subject_identifier"])


@receiver(
    post_save,
    sender=StockTransferItem,
    dispatch_uid="stock_transfer_item_on_post_save",
)
def stock_transfer_item_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        instance.stock.in_transit = True
        instance.stock.save(update_fields=["in_transit"])


@receiver(
    post_save,
    sender=ConfirmationAtLocationItem,
    dispatch_uid="confirm_at_location_item_on_post_save",
)
def confirm_at_location_item_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        instance.stock.confirmed_at_location = True
        instance.stock.save(update_fields=["confirmed_at_location"])


@receiver(
    post_save,
    sender=StorageBinItem,
    dispatch_uid="storage_bin_item_on_post_save",
)
def storage_bin_item_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        instance.stock.stored_at_location = True
        instance.stock.save(update_fields=["stored_at_location"])


@receiver(
    post_save,
    sender=DispenseItem,
    dispatch_uid="dispense_item_on_post_save",
)
def dispense_item_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
) -> None:
    if not raw and not update_fields:
        instance.stock.dispensed = True
        instance.stock.qty_out = 1
        instance.stock.unit_qty_out = instance.stock.container.qty * 1
        instance.stock.save(update_fields=["dispensed", "qty_out", "unit_qty_out"])
        StorageBinItem.objects.filter(stock=instance.stock).delete()


@receiver(post_delete, sender=ReceiveItem, dispatch_uid="receive_item_on_post_delete")
def receive_item_on_post_delete(sender, instance, using, **kwargs) -> None:
    instance.order_item.unit_qty_received = (
        instance.order_item.unit_qty_received - instance.unit_qty
    )
    instance.order_item.save()


@receiver(post_delete, sender=Stock, dispatch_uid="stock_on_post_delete")
def stock_on_post_delete(sender, instance, using, **kwargs) -> None:
    if getattr(instance, "confirmation", None):
        instance.confirm.delete()


@receiver(
    post_delete,
    sender=Allocation,
    dispatch_uid="allocation_post_delete",
)
def allocation_post_delete(sender, instance, using, **kwargs) -> None:
    if getattr(instance, "stock", None):
        instance.stock.subject_identifier = None
        instance.stock.save(update_fields=["subject_identifier"])


@receiver(
    post_delete,
    sender=StockTransferItem,
    dispatch_uid="stock_transfer_item_post_delete",
)
def stock_transfer_item_post_delete(sender, instance, using, **kwargs) -> None:
    instance.stock.in_transit = False
    instance.stock.save(update_fields=["in_transit"])


@receiver(
    post_delete,
    sender=ConfirmationAtLocationItem,
    dispatch_uid="confirm_at_location_item_post_delete",
)
def confirm_at_location_item_post_delete(sender, instance, using, **kwargs) -> None:
    instance.stock.confirmed_at_location = False
    instance.stock.save(update_fields=["confirmed_at_location"])


@receiver(
    post_delete,
    sender=StorageBinItem,
    dispatch_uid="storage_bin_item_post_delete",
)
def storage_bin_item_post_delete(sender, instance, using, **kwargs) -> None:
    instance.stock.stored_at_location = False
    instance.stock.save(update_fields=["stored_at_location"])


@receiver(
    post_delete,
    sender=DispenseItem,
    dispatch_uid="dispense_item_on_post_delete",
)
def dispense_item_on_post_delete(sender, instance, using, **kwargs) -> None:
    instance.stock.dispensed = False
    instance.stock.qty_out = 0
    instance.stock.unit_qty_out = 0
    instance.stock.save(update_fields=["dispensed", "qty_out", "unit_qty_out"])


@receiver(
    post_save,
    dispatch_uid="create_or_update_refills_on_post_save",
)
def create_or_update_refills_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
):
    if not raw:
        try:
            instance.related_visit_model_attr()  # see edc-visit-tracking
        except AttributeError:
            pass
        else:
            try:
                instance.creates_refills_from_crf()
            except AttributeError as e:
                if "creates_refills_from_crf" not in str(e):
                    raise
                pass


@receiver(
    post_save,
    dispatch_uid="update_refill_end_datetime",
)
def update_previous_refill_end_datetime_on_post_save(
    sender, instance, raw, created, update_fields, **kwargs
):
    if not raw and not update_fields and isinstance(instance, (StudyMedicationCrfModelMixin,)):
        update_previous_refill_end_datetime(instance)



# Generated by Django 3.2.7 on 2021-09-29 20:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_data_manager", "0020_auto_20210921_0403"),
    ]

    operations = [
        migrations.AddField(
            model_name="dataquery",
            name="auto_resolved",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="dataquery",
            name="missed_visit",
            field=models.BooleanField(
                default=False,
                help_text="If visit/timepoint was missed, data is not expected",
                verbose_name="Visit reported as missed",
            ),
        ),
        migrations.AddField(
            model_name="historicaldataquery",
            name="auto_resolved",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="historicaldataquery",
            name="missed_visit",
            field=models.BooleanField(
                default=False,
                help_text="If visit/timepoint was missed, data is not expected",
                verbose_name="Visit reported as missed",
            ),
        ),
    ]



# Generated by Django 4.2.6 on 2024-03-01 10:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_locator", "0036_alter_historicalsubjectlocator_site_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="consent_model",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="consent_version",
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="consent_model",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="consent_version",
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
    ]



from __future__ import annotations

import re

from clinicedc_constants import UUID_PATTERN
from django import forms
from django.urls.base import reverse
from django.utils.html import format_html

from edc_dashboard.url_names import url_names


class AlreadyConsentedFormMixin:
    def clean(self) -> dict:
        cleaned_data = super().clean()
        r = re.compile(UUID_PATTERN)
        if (
            self.instance.id
            and self.instance.subject_identifier
            and not r.match(self.instance.subject_identifier)
        ):
            url_name = url_names.get("subject_dashboard_url")
            url = reverse(
                url_name,
                kwargs={"subject_identifier": self.instance.subject_identifier},
            )
            raise forms.ValidationError(self.already_consented_validation_message(url))
        return cleaned_data

    def already_consented_validation_url(self, cleaned_data: dict | None = None) -> str:  # noqa: ARG002
        url_name = url_names.get("subject_dashboard_url")
        return reverse(
            url_name,
            kwargs={"subject_identifier": self.instance.subject_identifier},
        )

    def already_consented_validation_message(self, cleaned_data: dict | None = None) -> str:
        return format_html(
            'Not allowed. Subject has already consented. See subject <A href="{}">{}</A>',
            self.already_consented_validation_url(cleaned_data),
            self.instance.subject_identifier,
        )



from __future__ import annotations

from django import forms
from django.conf import settings

from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators import INVALID_ERROR, FormValidator
from edc_utils.text import convert_php_dateformat
from edc_visit_schedule.utils import raise_if_baseline

from ..diagnoses import ClinicalReviewBaselineRequired, Diagnoses, InitialReviewRequired


class ResultFormValidatorMixin(FormValidator):
    dx: tuple[str, str]  # e.g. (HIV, "HIV Infection")

    def clean(self):
        raise_if_baseline(self.cleaned_data.get("subject_visit"))
        try:
            raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        except ClinicalReviewBaselineRequired as e:
            self.raise_validation_error(str(e), INVALID_ERROR)
        try:
            self.validate_drawn_date_by_dx_date(*self.dx)
        except ClinicalReviewBaselineRequired as e:
            self.raise_validation_error(str(e), INVALID_ERROR)

    def validate_test_date_by_dx_date(
        self, prefix: str, dx_msg_label: str, test_date_fld: str | None = None
    ) -> None:
        return self.validate_drawn_date_by_dx_date(
            prefix=prefix, dx_msg_label=dx_msg_label, drawn_date_fld=test_date_fld
        )

    def validate_drawn_date_by_dx_date(
        self, prefix: str, dx_msg_label: str, drawn_date_fld: str | None = None
    ):
        drawn_date_fld = drawn_date_fld or "drawn_date"
        dx = Diagnoses(
            subject_visit=self.cleaned_data.get("subject_visit"),
            lte=True,
            limit_to_single_condition_prefix=prefix,
        )
        try:
            dx_date = dx.get_dx_date(prefix)
        except InitialReviewRequired:
            dx_date = None
        if not dx_date:
            raise forms.ValidationError(
                f"A {dx_msg_label} diagnosis has not been reported for this subject."
            )
        if dx_date > self.cleaned_data.get(drawn_date_fld):
            formatted_date = dx_date.strftime(
                convert_php_dateformat(settings.SHORT_DATE_FORMAT)
            )
            raise forms.ValidationError(
                {
                    "drawn_date": (
                        "Invalid. Subject was diagnosed with "
                        f"{dx_msg_label} on {formatted_date}."
                    )
                }
            )



from django.apps import apps as django_apps

from edc_auth.constants import ACCOUNT_MANAGER_ROLE
from edc_auth.site_auths import site_auths
from edc_export.constants import EXPORT

from .auth_objects import NOTIFICATION, codenames


def update_site_auths():
    site_auths.add_group(*codenames, name=NOTIFICATION)
    if django_apps.is_installed("edc_export"):
        site_auths.update_group("edc_notification.export_notification", name=EXPORT)
    site_auths.update_role(NOTIFICATION, name=ACCOUNT_MANAGER_ROLE)


update_site_auths()



class ModelAdminHideDeleteButtonOnCondition:
    def hide_delete_button_on_condition(self, request, object_id) -> bool:
        """Returns True if condition to hide button is met.

        Override.
        """
        return False

    def change_view(self, request, object_id, form_url="", extra_context=None):
        """Sets template context variable `show_delete` to False if
        `hide_delete_button_on_condition` returns True.
        """
        if self.hide_delete_button_on_condition(request, object_id):
            extra_context = extra_context or {}
            extra_context["show_delete"] = False
        elif extra_context:
            try:
                del extra_context["show_delete"]
            except KeyError:
                pass
        return super().change_view(
            request, object_id, form_url=form_url, extra_context=extra_context
        )



from django.conf import settings
from django.views.generic.base import TemplateView

from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from ..constants import CENTRAL_LOCATION
from .auths_view_mixin import AuthsViewMixin


class HomeView(EdcViewMixin, NavbarViewMixin, AuthsViewMixin, TemplateView):
    template_name = "edc_pharmacy/home.html"
    navbar_name = settings.APP_NAME
    navbar_selected_item = "pharmacy"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(CENTRAL_LOCATION=CENTRAL_LOCATION)
        return context



from __future__ import annotations

from typing import TYPE_CHECKING

import pandas as pd
from django.db.models import Q
from django_pandas.io import read_frame

from edc_appointment.analytics import get_appointment_df
from edc_appointment.constants import NEW_APPT
from edc_registration import get_registered_subject_model_cls
from edc_sites.site import sites as site_sites

from ...models import Rx

if TYPE_CHECKING:
    from ...models import StockRequest


def get_next_scheduled_visit_for_subjects_df(
    stock_request: StockRequest | None = None,
) -> pd.DataFrame:
    # get the next scheduled visit from appointment
    # subject_identifier, next_visit_code, next_appt_datetime
    df_appt = get_appointment_df(
        normalize=True,
        localize=True,
        values=[
            "id",
            "subject_identifier",
            "visit_code",
            "visit_code_sequence",
            "site",
            "appt_datetime",
            "appt_status",
            "appt_type",
        ],
        site_id=int(stock_request.location.site_id) if stock_request else None,
    )
    if df_appt.empty:
        df = pd.DataFrame()
    else:
        if stock_request:
            if stock_request.start_datetime:
                df_appt = df_appt[
                    df_appt.next_appt_datetime
                    >= pd.Timestamp(stock_request.start_datetime.date()).to_datetime64()
                ]
                df_appt = df_appt.reset_index(drop=True)
            if stock_request.cutoff_datetime:
                df_appt = df_appt[
                    df_appt.next_appt_datetime
                    <= pd.Timestamp(stock_request.cutoff_datetime.date()).to_datetime64()
                ]
                df_appt = df_appt.reset_index(drop=True)
        # get the first appointment due
        df = (
            df_appt[(df_appt.appt_status == NEW_APPT) & (df_appt.visit_code_sequence == 0)]
            .sort_values(by=["appt_datetime"])
            .groupby(by=["subject_identifier"])
            .first()
        )
        df = df.reset_index()
        df = df.sort_values(by=["next_appt_datetime"])
        df = df.reset_index(drop=True)

        # merge with registered_subject
        df_registered_subject = read_frame(
            get_registered_subject_model_cls().objects.values("id", "subject_identifier"),
            verbose=False,
        )
        df_registered_subject = df_registered_subject.rename(
            columns={"id": "registered_subject_id"}
        )
        df = df.merge(
            df_registered_subject,
            on="subject_identifier",
            how="left",
            suffixes=("", "_y"),
        )
        df = df.reset_index(drop=True)

        if stock_request:
            # merge with prescription
            df_rx = read_frame(
                Rx.objects.values(
                    "id",
                    "registered_subject__subject_identifier",
                    "rx_expiration_date",
                    "rando_sid",
                ).filter(
                    (
                        Q(rx_expiration_date__gte=stock_request.request_datetime.date())
                        | Q(rx_expiration_date__isnull=True)
                    ),
                    medications__in=[stock_request.formulation.medication],
                )
            )

            df_rx = df_rx.rename(
                columns={
                    "registered_subject__subject_identifier": "subject_identifier",
                    "id": "rx_id",
                }
            )
            df = df.merge(
                df_rx[["subject_identifier", "rx_id", "rando_sid"]],
                on="subject_identifier",
                how="left",
                suffixes=("", "_y"),
            )
            df = df[df.rx_id.notna()]
            df = df.reset_index(drop=True)

            df["site_name"] = df["site_id"].apply(lambda x: site_sites.get(x).name)

            if stock_request.subject_identifiers:
                subject_identifiers = stock_request.subject_identifiers.split("\n")
                df = df[df.subject_identifier.isin(subject_identifiers)]
                df = df.reset_index(drop=True)
            elif stock_request.excluded_subject_identifiers:
                excluded_subject_identifiers = (
                    stock_request.excluded_subject_identifiers.split("\n")
                )
                excluded_subject_identifiers = [
                    s.strip() for s in excluded_subject_identifiers
                ]
                df = df[~df.subject_identifier.isin(excluded_subject_identifiers)]
                df = df.reset_index(drop=True)
    return df



from __future__ import annotations

from django import forms

from ..exceptions import VisitScheduleNonCrfModelFormMixinError
from ..schedule import Schedule
from ..site_visit_schedules import SiteVisitScheduleError, site_visit_schedules
from ..subject_schedule import (
    NotOnScheduleError,
    NotOnScheduleForDateError,
    SubjectSchedule,
)
from ..utils import report_datetime_within_onschedule_offschedule_datetimes
from ..visit_schedule import VisitSchedule

__all__ = ["VisitScheduleNonCrfModelFormMixin"]


class VisitScheduleNonCrfModelFormMixin:
    """A ModelForm mixin for non-CRFs to validate that the subject is
    onschedule at the time of the report.
    """

    # loss_to_followup_model, offschedule_model, onschedule_model
    get_by_model_attr: str | None = None
    offschedule_compare_dates_as_datetimes = True

    def clean(self):
        cleaned_data = super().clean()
        self.is_onschedule_or_raise()
        self.report_datetime_within_schedule_datetimes()
        return cleaned_data

    @property
    def visit_schedule_name(self) -> str:
        return self.visit_schedule.name

    @property
    def schedule_name(self) -> str:
        return self.schedule.name

    @property
    def visit_schedule(self) -> VisitSchedule:
        visit_schedule = getattr(self.instance, "visit_schedule", None)
        if not visit_schedule:
            if self.get_by_model_attr:
                visit_schedule, _ = site_visit_schedules.get_by_model(
                    attr=self.get_by_model_attr,
                    model=self._meta.model._meta.label_lower,
                )
            else:
                raise VisitScheduleNonCrfModelFormMixinError(
                    "Unable to determine `visit schedule`. "
                    f"See model and modelform for {self._meta.model}."
                )
        return visit_schedule

    @property
    def schedule(self) -> Schedule:
        try:
            _, schedule = site_visit_schedules.get_by_model(
                attr=self.get_by_model_attr, model=self._meta.model._meta.label_lower
            )
        except SiteVisitScheduleError:
            schedule = self.instance.schedule
        return schedule

    def is_onschedule_or_raise(self) -> None:
        if self.report_datetime:
            subject_schedule = SubjectSchedule(
                self.get_subject_identifier(), self.visit_schedule, self.schedule
            )
            try:
                subject_schedule.onschedule_or_raise(
                    report_datetime=self.report_datetime,
                    compare_as_datetimes=self.offschedule_compare_dates_as_datetimes,
                )
            except (NotOnScheduleError, NotOnScheduleForDateError) as e:
                raise forms.ValidationError(str(e)) from e

    def report_datetime_within_schedule_datetimes(self) -> None:
        if self.report_datetime:
            report_datetime_within_onschedule_offschedule_datetimes(
                subject_identifier=self.get_subject_identifier(),
                report_datetime=self.report_datetime,
                visit_schedule_name=self.visit_schedule_name,
                schedule_name=self.schedule_name,
                exception_cls=forms.ValidationError,
            )



# Generated by Django 5.1.2 on 2024-11-11 14:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pylabels", "0008_labelconfiguration_requires_allocated"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="labelconfiguration",
            name="requires_allocated",
        ),
        migrations.AddField(
            model_name="labelconfiguration",
            name="requires_allocation",
            field=models.BooleanField(default=False, verbose_name="Is a subject label"),
        ),
    ]



from django import forms

from ..models import Aliquot


class AliquotForm(forms.ModelForm):
    aliquot_identifier = forms.CharField(label="Aliquot identifier", disabled=True)

    class Meta:
        model = Aliquot
        fields = "__all__"



# Generated by Django 3.0.4 on 2020-05-11 23:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_export", "0005_exportdata_importdata"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="exportdata",
            options={"permissions": [("show_export_admin_action", "Show export action")]},
        ),
        migrations.AlterModelOptions(
            name="importdata",
            options={"permissions": [("show_import_admin_action", "Show import action")]},
        ),
        migrations.AlterModelOptions(
            name="plan",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
            },
        ),
    ]



from collections import OrderedDict

from django.apps import apps as django_apps
from django.conf import settings
from django.contrib import messages
from django.contrib.admin import sites
from django.core.exceptions import ObjectDoesNotExist
from edc_auth.is_custom_permissions_model import is_custom_permissions_model
from edc_randomization.utils import is_randomization_list_model

from .constants import EXPORT
from .model_options import ModelOptions

__all__ = ["ExportableModelsForUser"]


class Exportable(OrderedDict):
    def __init__(self, app_config=None):
        super().__init__()
        self._inlines = {}
        self.app_config = app_config
        self.historical_models = []
        self.list_models = []
        self.models = []
        self.name = app_config.name
        self.verbose_name = app_config.verbose_name
        for model in app_config.get_models():
            if is_randomization_list_model(model=model) or is_custom_permissions_model(
                model=model
            ):
                continue
            model_opts = ModelOptions(model=model._meta.label_lower)
            if model_opts.is_historical:
                self.historical_models.append(model_opts)
            elif model_opts.is_list_model:
                self.list_models.append(model_opts)
            elif not model._meta.proxy:
                self.models.append(model_opts)
        self.models.sort(key=lambda x: x.verbose_name.title())
        self.historical_models.sort(key=lambda x: x.verbose_name.title())
        self.list_models.sort(key=lambda x: x.verbose_name.title())
        self.inlines = self.get_inlines(app_config.name)
        self.update(
            dict(
                models=self.models,
                historical_models=self.historical_models,
                list_models=self.list_models,
                inlines=self.inlines,
            )
        )

    def __repr__(self):
        return f"{self.__class__.__name__}(app_config={self.app_config})"

    def __str__(self):
        return self.name

    def get_inlines(self, app_label):
        if not self._inlines:
            for site in sites.all_sites:
                for model_cls, admin_site in site._registry.items():
                    for inline_cls in admin_site.inlines:
                        model_opts = ModelOptions(model=inline_cls.model._meta.label_lower)
                        try:
                            self._inlines[model_cls._meta.app_label].append(model_opts)
                        except KeyError:
                            self._inlines[model_cls._meta.app_label] = [model_opts]
                        self._inlines[model_cls._meta.app_label].sort(
                            key=lambda x: x.verbose_name.title()
                        )
        return self._inlines.get(app_label)


class ExportableModelsForUser(OrderedDict):
    """A dictionary-like object that creates a "list" of
    models, historical models, and list models that may be exported.

    Checks each `AppConfig.has_exportable_data` and if True
    includes that apps models, including historical and list models.
    """

    export_group_name = EXPORT

    default_app_labels = getattr(
        settings,
        "EDC_EXPORTABLE_DEFAULT_APPS",
        ["sites", "auth", "admin"],
    )

    def __init__(self, app_configs=None, user=None, request=None):
        super().__init__()
        app_configs = app_configs or self.get_app_configs()
        app_configs.sort(key=lambda x: x.verbose_name)
        try:
            user.groups.get(name=self.export_group_name)
        except ObjectDoesNotExist:
            messages.error(request, "You do not have sufficient permissions to export data.")
        else:
            for app_config in app_configs:
                self.update({app_config.name: Exportable(app_config=app_config)})

    def get_app_configs(self):
        """Returns a list of app_configs with exportable data."""
        app_configs = []
        for app_config in django_apps.get_app_configs():
            try:
                has_exportable_data = app_config.has_exportable_data
            except AttributeError:
                has_exportable_data = None
            if has_exportable_data:
                app_configs.append(app_config)
        for app_label in self.default_app_labels:
            app_configs.append(django_apps.get_app_config(app_label))
        return app_configs



import contextlib

from celery import current_app
from celery.result import AsyncResult
from django.conf import settings
from kombu.exceptions import OperationalError

celery_enabled = getattr(settings, "CELERY_ENABLED", False)


def run_task_sync_or_async(task, *args, **kwargs) -> AsyncResult:
    """Run a task with celery if running"""

    if not celery_enabled or not celery_is_active() or current_app.conf.task_always_eager:
        return task(*args, **kwargs)
    return task.delay(*args, **kwargs)


def celery_is_active() -> dict:
    """Inspect if workers are running, e.g was celery
    service started.
    """
    i = current_app.control.inspect()
    try:
        return i.active()
    except OperationalError:
        return {}


def get_task_result(obj) -> AsyncResult | None:
    """Query celery task and return result or None

    If celery not running will raise an exception
    that is caught here.
    """
    result = None
    if obj.task_id:
        with contextlib.suppress(TypeError, ValueError):
            result = AsyncResult(str(obj.task_id))
    return result


__all__ = ["celery_is_active", "get_task_result", "run_task_sync_or_async"]



from clinicedc_constants import HIGH_PRIORITY

from edc_action_item.action_with_notification import ActionWithNotification
from edc_adverse_event.constants import DEATH_REPORT_ACTION
from edc_ltfu.constants import LTFU_ACTION
from edc_unblinding.constants import UNBLINDING_REVIEW_ACTION

from .constants import END_OF_STUDY_ACTION
from .utils import get_offstudy_model


class EndOfStudyAction(ActionWithNotification):
    admin_site_name = None  # "inte_prn_admin"

    name = END_OF_STUDY_ACTION
    display_name = "Submit End of Study Report"
    notification_display_name = "End of Study Report"
    parent_action_names = (
        UNBLINDING_REVIEW_ACTION,
        DEATH_REPORT_ACTION,
        LTFU_ACTION,
    )
    show_link_to_changelist = True
    priority = HIGH_PRIORITY
    singleton = True

    @classmethod
    def get_reference_model(cls) -> str:
        """Returns the reference model label lower.

        Overridden because site_visit_schedule needs to
        register visit schedules first.
        """
        return get_offstudy_model()



# Generated by Django 3.2.13 on 2022-09-29 14:42

from django.db import migrations, models

import edc_sites.models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0015_auto_20220913_2139"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="product",
            managers=[
                ("objects", models.Manager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="stock",
            managers=[
                ("objects", models.Manager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
    ]



from clinicedc_constants import CLOSED, FEEDBACK, HIGH_PRIORITY, NEW, OPEN, RESOLVED
from django import forms

from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_form_validators import FormValidator, FormValidatorMixin

from ..constants import CLOSED_WITH_ACTION
from ..models import DataQuery


class DataQueryFormValidator(FormValidator):
    def clean(self):
        if self.cleaned_data.get("data_dictionaries"):
            models = list(
                set([obj.model for obj in self.cleaned_data.get("data_dictionaries")])
            )
            if len(models) > 1:
                raise forms.ValidationError(
                    {"data_dictionaries": "Invalid. Select questions from one CRF only"}
                )

        self.required_if(
            HIGH_PRIORITY,
            field="query_priority",
            field_required="recipients",
            inverse=False,
        )

        self.required_if_not_none(
            field="visit_schedule",
            field_required="visit_code_sequence",
            field_required_evaluate_as_int=True,
        )

        # Site
        self.required_if(
            RESOLVED,
            field="site_response_status",
            field_required="site_resolved_datetime",
        )
        self.required_if(
            OPEN,
            FEEDBACK,
            RESOLVED,
            field="site_response_status",
            field_required="site_response_text",
        )

        if self.cleaned_data.get("site_response_status") in [
            NEW,
            OPEN,
            FEEDBACK,
        ] and self.cleaned_data.get("status") in [CLOSED, CLOSED_WITH_ACTION]:
            raise forms.ValidationError({"status": "Invalid: Site response is not resolved."})

        # DM
        self.required_if(
            CLOSED,
            CLOSED_WITH_ACTION,
            field="status",
            field_required="resolved_datetime",
        )
        self.required_if(CLOSED, CLOSED_WITH_ACTION, field="status", field_required="dm_user")
        self.required_if(
            CLOSED_WITH_ACTION,
            field="status",
            field_required="plan_of_action",
            inverse=False,
        )
        self.required_if(True, field="locked", field_required="locked_reason")


class DataQueryForm(ActionItemFormMixin, FormValidatorMixin, forms.ModelForm):
    form_validator_cls = DataQueryFormValidator

    class Meta:
        model = DataQuery
        fields = "__all__"



# Generated by Django 5.1.2 on 2024-11-15 04:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0025_alter_stock_receive_item"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalstockrequest",
            name="cutoff_datetime",
            field=models.DateTimeField(
                blank=True, default=None, help_text="May be left blank", null=True
            ),
        ),
        migrations.AddField(
            model_name="stockrequest",
            name="cutoff_datetime",
            field=models.DateTimeField(
                blank=True, default=None, help_text="May be left blank", null=True
            ),
        ),
    ]



from __future__ import annotations

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from edc_model.validators import datetime_not_future
from edc_protocol.validators import datetime_not_before_study_start
from edc_visit_schedule.model_mixins import CrfScheduleModelMixin

from ..base import VisitMethodsModelMixin


class VisitTrackingCrfModelMixin(VisitMethodsModelMixin, CrfScheduleModelMixin, models.Model):
    """Base mixin for all CRF models (used by edc-crf CrfModelMixin).

    CRFs have a OneToOne relation to the related visit model

    Assumes `subject_visit` is the related visit model field attr
    on the Index.

    See also: edc_crf CrfModelMixin
    """

    # assuming subject_visit
    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL, on_delete=models.PROTECT
    )

    report_datetime = models.DateTimeField(
        verbose_name=_("Report Date"),
        validators=[datetime_not_before_study_start, datetime_not_future],
        default=timezone.now,
        help_text=_(
            "If reporting today, use today's date/time, otherwise use "
            "the date/time this information was reported."
        ),
    )

    @property
    def subject_identifier(self) -> str | None:
        try:
            return self.related_visit.subject_identifier
        except AttributeError as e:
            if "subject_identifier" not in str(e):
                raise
            return None

    @classmethod
    def related_visit_model_attr(cls) -> str:
        # assuming subject_visit
        return "subject_visit"

    class Meta:
        abstract = True
        # assuming subject_visit
        indexes = (
            models.Index(fields=["subject_visit", "site", "id"]),
            models.Index(fields=["subject_visit", "report_datetime"]),
        )



from django.contrib.sites.models import Site
from django.db import models
from django.db.models import DO_NOTHING, Index
from django.utils import timezone

from ..utils import recreate_db_view
from .qa_reports_permissions import qa_reports_permissions


class QaReportModelMixin(models.Model):
    report_model = models.CharField(max_length=50)

    subject_identifier = models.CharField(max_length=25)

    site = models.ForeignKey(Site, on_delete=DO_NOTHING)

    created = models.DateTimeField(default=timezone.now)

    def __str__(self):
        return f"{self.subject_identifier}: {self.report_model}"

    @classmethod
    def recreate_db_view(cls, drop: bool | None = None, verbose: bool | None = None):
        recreate_db_view(cls, drop=drop, verbose=verbose)

    class Meta:
        abstract = True
        default_permissions = qa_reports_permissions
        indexes = (Index(fields=["subject_identifier", "site"]),)
